@@ -1,14 +1,20 @@
 from collections import defaultdict
+import functools
+import json
+import operator
 from pathlib import Path
+import platform
 import re
+import subprocess
 import sys
 from typing import Dict
+import uuid
 from uuid import UUID
 
 import attr
 import boto3
+from botocore.exceptions import WaiterError
 import click
-from ffprobe import FFProbe
 
 from boiler import BoilerSession, cli
 from boiler.commands.utils import exit_with, handle_request_error
@@ -23,6 +29,10 @@ video_name_regexp = re.compile(
 )
 BUCKET = 'diva-mturk-ingestion-prod'
 
+ELASTIC_TRANSCODER_PIPELINE_ID = '1536759483093-n34917'
+ELASTIC_TRANSCODER_PORTRAIT_PRESET_ID = '1582841064651-nyezpa'
+ELASTIC_TRANSCODER_LANDSCAPE_PRESET_ID = '1536756735287-cx0qtx'
+
 
 def bold(x, **kwargs):
     return click.style(x, bold=True, **kwargs)
@@ -35,17 +45,36 @@ class VideoInfo:
     width: int
     height: int
     name: str
+    portrait: bool
 
     @classmethod
     def create(cls, file: str) -> 'VideoInfo':
-        probe = FFProbe(file)
-        stream = probe.streams[0]
+        ffprobe = 'ffprobe'
+        if platform.system() == 'Windows':
+            ffprobe = 'ffprobe.exe'
+        try:
+            ps = subprocess.Popen(
+                [ffprobe, '-print_format', 'json', '-show_streams', file],
+                stdout=subprocess.PIPE,
+                stderr=subprocess.DEVNULL,
+            )
+        except FileNotFoundError:
+            raise click.UsageError(f'Could not find {ffprobe} in your PATH, try install ffmpeg.')
+
+        output = ps.communicate()[0]
+        stream = json.loads(output)['streams'][0]
+        stream['framerate'] = round(
+            functools.reduce(
+                operator.truediv, map(int, stream.get('avg_frame_rate', '').split('/'))
+            )
+        )
         return cls(
-            frame_rate=stream.framerate,
-            duration=float(stream.duration),
-            width=int(stream.width),
-            height=int(stream.height),
+            frame_rate=stream['framerate'],
+            duration=float(stream['duration']),
+            width=int(stream['width']),
+            height=int(stream['height']),
             name=str(Path(file).stem),
+            portrait=int(stream['height']) > int(stream['width']),
         )
 
     def s3_path(self, id: UUID) -> str:
@@ -61,50 +90,76 @@ class VideoInfo:
         return f'uuid-encoded/{date}/{time}/{m["gtag"]}_{m["location"]}/{id}.mp4'
 
 
-def ingest_video(video_path: str, release_batch: str, session: BoilerSession):
-    video_name = Path(video_path).stem
-    # until we fix the lambda, transcoding needs to be done manually:
-    # ffmpeg -i <input> -f lavfi -i anullsrc -c:a aac -shortest -threads 12 \
-    #        -c:v h264 -profile high <output>.avi
+def _transcode_video(upload_id: str, video_info: VideoInfo):
+    transcoder = boto3.client('elastictranscoder', region_name='us-east-1')
+    job = transcoder.create_job(
+        PipelineId=ELASTIC_TRANSCODER_PIPELINE_ID,
+        Input={'Key': f'uploads/original/{upload_id}', 'FrameRate': 'auto'},
+        Output={
+            'Key': f'uploads/encoded/{upload_id}',
+            'PresetId': ELASTIC_TRANSCODER_PORTRAIT_PRESET_ID
+            if video_info.portrait
+            else ELASTIC_TRANSCODER_LANDSCAPE_PRESET_ID,
+        },
+    )
 
-    if Path(video_path).suffix != '.mp4':
-        raise Exception('Expected a transcoded video')
+    try:
+        transcoder.get_waiter('job_complete').wait(Id=job['Job']['Id'], WaiterConfig={'Delay': 10})
+    except WaiterError:
+        raise Exception(f'an error occurred while transcoding {upload_id}')
+
+
+def ingest_video(video_path: str, release_batch: str, phase: str, session: BoilerSession):
+    video_name = Path(video_path).stem
+    video_info = VideoInfo.create(video_path)
+    upload_id = str(uuid.uuid4())
+
+    if Path(video_path).suffix != '.avi':
+        raise Exception('Expected a non-transcoded video')
 
     r = session.get('video', json={'name': video_name})
-    out = handle_request_error(r)
-    if not r.ok:
-        raise Exception(f'Could not query server for video={video_name}')
+    r.raise_for_status()
 
-    if len(out['response']):
-        video_id = out['response'][0]['id']
+    if r.json():
         click.echo(f'name={video_name} already exists', err=True)
-    else:
-        post_data = {'video_name': video_name, 'release_batch': release_batch}
-        r = session.post('video/name', json=post_data)
-        if not r.ok:
-            raise Exception(f'Could not generate new video entity for {video_name}')
-        video_id = r.json()['id']
+        return
 
-    click.echo(f'id={video_id} created for name={video_name}', err=True)
-    click.echo(f'sending name={video_name} id={video_id} to S3', err=True)
-
-    s3 = boto3.client('s3')
-    info = VideoInfo.create(video_path)
     try:
-        s3.upload_file(video_path, BUCKET, info.s3_path(video_id))
+        s3 = boto3.client('s3')
+        click.echo(f'uploading {video_name}..')
+        s3.upload_file(video_path, BUCKET, f'uploads/original/{upload_id}')
     except Exception:
         click.echo('Could not upload file to S3.  Have you set up your credentials?', err=True)
         raise
 
-    patch_data = {
-        'duration': info.duration,
-        'frame_rate': info.frame_rate,
-        'height': info.height,
-        'width': info.width,
-    }
-    resp = session.patch(f'video/{video_id}', json=patch_data)
+    click.echo(f'transcoding {video_name}..')
+    _transcode_video(upload_id, video_info)
+
+    # create stumpf video and move encoded upload to correct location
+    r = session.post('video/name', json={'video_name': video_name, 'release_batch': release_batch})
+    if not r.ok:
+        raise Exception(f'Could not generate new video for {video_name}')
+
+    src_path = f'uploads/encoded/{upload_id}'
+    dest_path = f'{video_info.s3_path(r.json()["id"])}'
+    s3.copy_object(Bucket=BUCKET, CopySource={'Bucket': BUCKET, 'Key': src_path}, Key=dest_path)
+    s3.delete_object(Bucket=BUCKET, Key=f'uploads/original/{upload_id}')
+    s3.delete_object(Bucket=BUCKET, Key=src_path)
+
+    resp = session.patch(
+        f'video/{r.json()["id"]}',
+        json={
+            'duration': video_info.duration,
+            'frame_rate': video_info.frame_rate,
+            'height': video_info.height,
+            'width': video_info.width,
+            'phase': phase,
+            'path': f'https://s3.amazonaws.com/{BUCKET}/{dest_path}',
+        },
+    )
     if not resp.ok:
         raise Exception(f'Could not set video metadata on {video_path}')
+    click.echo(f'ingested {video_name} -> {r.json()["id"]}.')
 
 
 @click.group(name='video', short_help='ingest and query video')
@@ -193,19 +248,42 @@ def search(ctx, **kwargs):
 @click.option(
     '--release-batch', type=click.Choice([e.value for e in ReleaseBatches]), required=True
 )
+@click.option(
+    '--phase', type=click.Choice(['m1', 'm2', 'meva', 'none', 'BWC', 'm2-boost']), required=True
+)
 @click.pass_obj
-def add(ctx, videos, release_batch):
+def add(ctx, videos, release_batch, phase):
     """Ingest locally transcoded video files into Stumpf.
 
     \b
-    Reference encoding settings are as follows:
-        ffmpeg -i <input> -f lavfi -i anullsrc -c:a aac -shortest \\
-               -c:v h264 -profile high <output>.avi
+    Videos are found in the following SMB shares:
+    * //diva-nas06/m1-complete   (all m1 locations)
+    * //diva-nas01/m2-albany     (alb)
+    * //diva-nas03/m2n-keu       (gdb, Greo1)
+    * //diva-nas05/m2-unimore    (uni)
+
+    Mount with:
+    \b
+    mount -t cifs -o username=<user>,password=<password>,ro <share> <local>
 
     This command requires a working `ffprobe` in your path.
     """
+    phase = None if phase == 'none' else phase
+
     for video in videos:
-        ingest_video(video, release_batch, ctx['session'])
+        ingest_video(video, release_batch, phase, ctx['session'])
+
+
+@video.command(name='summary', short_help='get a summary of all videos')
+@click.argument('output', type=click.File('wb'), default='-')
+@click.pass_obj
+def get_video_summary(ctx, output):
+    """
+    Get a summary of all videos in csv format.  By default, this will print to
+    the terminal.  Add a path argument to write to a file.
+    """
+    r = ctx['session'].get('video/summary')
+    output.write(r.content)
 
 
 cli.add_command(video)
