@@ -7,6 +7,7 @@ They are only useful internally to this library, and are intended as a
 translation layer for data either before or after it exists within the stumpf
 system.
 """
+import logging
 from typing import Any, Dict, Iterable, Iterator, List, NamedTuple, Optional, Tuple
 
 import attr
@@ -14,6 +15,8 @@ import attr
 from boiler import BoilerError
 from boiler.definitions import ActivityPipelineStatuses, ActivityType, ActorType
 
+logger = logging.getLogger(__name__)
+
 
 def sort_detections(detections: List['Detection']) -> List['Detection']:
     return sorted(detections, key=lambda d: d.frame)
@@ -49,7 +52,7 @@ class Box(NamedTuple):
 
     @property
     def width(self) -> int:
-        return self.left - self.right
+        return self.right - self.left
 
     @property
     def height(self) -> int:
@@ -81,6 +84,26 @@ class Box(NamedTuple):
     def from_dict(cls, d: Dict[str, Any]) -> 'Box':
         return cls(left=d['left'], right=d['right'], top=d['top'], bottom=d['bottom'])
 
+    def fix_order(self) -> 'Box':
+        left = self.left
+        right = self.right
+        top = self.top
+        bottom = self.bottom
+
+        if left > right:
+            left, right = right, left
+        elif left == right:
+            logger.warning('Zero width bounding box encountered')
+            right = left + 1
+
+        if top > bottom:
+            top, bottom = bottom, top
+        elif bottom == top:
+            logger.warning('Zero height bounding box encountered')
+            bottom = top + 1
+
+        return Box(left=left, right=right, top=top, bottom=bottom)
+
 
 @attr.s(auto_attribs=True, kw_only=True, frozen=True)
 class Detection:
@@ -97,6 +120,9 @@ class Detection:
         box = self.box.interpolate(other.box, distance)
         return Detection(frame=frame, box=box, keyframe=False)
 
+    def fix_order(self) -> 'Detection':
+        return attr.evolve(self, box=self.box.fix_order())
+
 
 @attr.s(auto_attribs=True, kw_only=True, frozen=True, hash=False)
 class Actor:
@@ -120,6 +146,50 @@ class Actor:
             first_detection = self.detections[0]
         return hash((self.actor_type.value, self.begin, self.end, first_detection))
 
+    def inject_keyframe(self, frame: int) -> 'Actor':
+        """Return the actor with a new keyframe linearly interpolated.
+
+        If the frame is already a keyframe or if the frame is outside of the
+        valid range [begin, end], then no change will be made.
+        """
+        new_detections: List[Detection] = []
+        for index in range(len(self.detections) - 1):
+            d1 = self.detections[index]
+            d2 = self.detections[index + 1]
+            new_detections.append(d1)
+
+            if d1.frame == frame:
+                return self
+            elif d1.frame < frame < d2.frame:
+                interpolated = attr.evolve(d1.interpolate(d2, frame), keyframe=True)
+                new_detections.append(interpolated)
+
+        new_detections.append(self.detections[-1])
+        return attr.evolve(self, detections=new_detections)
+
+    def truncate_framerange(self) -> 'Actor':
+        """Return the actor with frame range truncated to the detection range."""
+        begin = max(self.begin, self.detections[0].frame)
+        end = min(self.end, self.detections[-1].frame)
+        return attr.evolve(self, begin=begin, end=end)
+
+    def truncate_detections(self) -> 'Actor':
+        return attr.evolve(
+            self, detections=[d for d in self.detections if self.begin <= d.frame <= self.end]
+        )
+
+    @property
+    def interpolated_detections(self) -> Iterator[Detection]:
+        if not self.detections:
+            return
+        last_detection = self.detections[0]
+        yield last_detection
+        for detection in self.detections[1:]:
+            for frame in range(last_detection.frame + 1, detection.frame):
+                yield last_detection.interpolate(detection, frame)
+            yield detection
+            last_detection = detection
+
 
 @attr.s(auto_attribs=True, kw_only=True, frozen=True)
 class Activity:
@@ -134,6 +204,27 @@ class Activity:
         actors = [a.pruned() for a in self.actors]
         return attr.evolve(self, actors=actors)
 
+    def truncate(self) -> 'Activity':
+        """Remove extra detections and truncate frame range."""
+        if not self.actors:
+            logger.warning('Activity does not contain any tracks')
+            return self
+
+        new_actors = [
+            a.inject_keyframe(a.begin)
+            .inject_keyframe(a.end)
+            .truncate_framerange()
+            .truncate_detections()
+            for a in self.actors
+        ]
+        begin = self.actors[0].begin
+        end = self.actors[0].end
+        for actor in self.actors[1:]:
+            begin = min(begin, actor.begin)
+            end = max(end, actor.end)
+
+        return attr.evolve(self, begin=begin, end=end, actors=new_actors)
+
 
 @attr.s(auto_attribs=True, kw_only=True, frozen=True)
 class ActivityList:
