@@ -168,8 +168,8 @@ def serialize_types(actor_map: Dict[int, Actor]):
 def serialize_geom(actor_map: Dict[int, Actor]):
     geom_id = 1
     for actor_id, a in actor_map.items():
-        for d in a.detections:
-            if d.keyframe:
+        for d in a.interpolated_detections:
+            if a.begin <= d.frame <= a.end:
                 line = {
                     'geom': {
                         ACTOR_ID: actor_id,
@@ -177,7 +177,7 @@ def serialize_geom(actor_map: Dict[int, Actor]):
                         FRAME: d.frame,
                         # TODO ts1 requires framerate
                         BOX: f'{d.box.left} {d.box.top} {d.box.right} {d.box.bottom}',
-                        KEYFRAME: True,
+                        KEYFRAME: d.keyframe,
                     }
                 }
                 geom_id = geom_id + 1
@@ -185,8 +185,6 @@ def serialize_geom(actor_map: Dict[int, Actor]):
 
 
 def serialize_activities(activity_map: Dict[int, Activity], actor_map: Dict[int, Actor]):
-    inverse_actor_map = {actor: actor_id for actor_id, actor in actor_map.items()}
-
     for activity_id, activity in activity_map.items():
         activity = activity_map[activity_id]
 
@@ -197,7 +195,7 @@ def serialize_activities(activity_map: Dict[int, Activity], actor_map: Dict[int,
                 TIMESPANS: [{FRAME_TIMESPAN: [activity.begin, activity.end]}],
                 ACTORS: [
                     {
-                        ACTOR_ID: inverse_actor_map[actor],
+                        ACTOR_ID: actor.actor_id,
                         TIMESPANS: [{FRAME_TIMESPAN: [actor.begin, actor.end]}],
                     }
                     for actor in activity.actors
